/**
 * Integracja AngularJS z instniejącym już systemu maskowania wprowadzanych danych (maskDefinitions.js).
 */
ttUI.directive('ttUiMask', function() {
	// czy można z argumentu parsować numer
	// czy przypadkiem nie jest undefined, null, ''
	function isParsableNumber(value) {
		return ((value && value != null && value != '') ? true : false); 
	}
	
	return {
		require : 'ngModel',
		link: function($scope, iElement, iAttrs, controller) {
			controller.$render = function () {
				var value = controller.$viewValue || '';
				iElement.val(value);

				switch(iAttrs.ttUiMask) {
					case 'data':
						iElement.attr("size", "11");
						iElement.layoutDate();
						iElement.not("[readonly]").datepicker({
					  			showOn			: "both", 
					  			buttonImage		: root+"/img/calendar.gif",
					   			yearRange		: '-85y:+15y',
					   			buttonImageOnly	: true,
					   			dateFormat		: 'yy-mm-dd',
								onSelect		: function (dateText, inst) {
													$scope.$apply(function(){
														controller.$setViewValue(dateText);
													});
												  }
						});
						break;
						
					case 'iban':
						iElement.iban();
						break;
						
					case 'doublePrecision':
						iElement.doublePrecision();
						break;
					
					default:
						iElement.setMask({
							mask : iAttrs.ttUiMask,
							defaultValue : '',
							selectCharsOnFocus : false
						});
				}
			};
			
			switch(iAttrs.ttUiMask) {
				case 'doublePrecision':
					// czytanie z widoku
					controller.$parsers.push(
				            function(viewValue) {
				            	return (isParsableNumber(viewValue) ? parseFloat(viewValue.replace(',', '.')) : viewValue);
				            }
				        );
					break;
				
				default:
				break;
			}
			
			// wymuszenie aktualizacji modelu po zmianie wartości przez użytkownika i maskowanie
			iElement.bind('keyup', function () {
				$scope.$apply(function () {
					controller.$setViewValue(iElement.maskedVal());
				});
			});
        }
	};
});

/**
 * Pytanie o potwierdzenie wykonania operacji przez program
 * (oczywiście w razie zajścia takiej potrzeby).
 */
ttUI.directive('ttUiConfirmOperation', function() {
    return {
    	link: function($scope, iElement, iAttrs, controller) {
        	var info = iAttrs.uiTtConfirmOperation || 'Potwierdź przeprowadzenie działania...';
        	
        	iElement.click(function(event) {
            	var odp = confirm(info);
            	
            	if (odp) {
            		pokazLoadImage();
            		
            		return true;
            	} else {
                    event.preventDefault();
                    
            		return false;
            	}
            });
        }
    };
});

/**
 * Formularz, który jest wysyłany w tle bez przeładowywania strony
 */
ttUI.directive('ttUiAjaxForm', function() {
	// dla celów właściwej identyfikacji
	function _escape(text) {
		if(typeof text != 'undefined') {
			return text.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&");
		}
		
		return '';
	}
	
	// przetwarzanie błędów walidacji (domyślne)
	function showErrorResponse(response, statusText, xhr, $form) {
		var errorJsonObj = $.parseJSON(response.responseText);

		for(var errorKey in errorJsonObj) {
			var $errorContainer = $("#" + _escape(errorKey));
			
			$errorContainer.html(errorJsonObj[errorKey]);
			$errorContainer.show();
		}
	}
	
	return {
		controller: ['$scope', function ttUiAjaxFormController() {
		    this.options = {
				type: 'POST',
		    };
		}],
		priority: 499,
		require: 'ttUiAjaxForm',
		link: function($scope, iElement, iAttrs, controller) {
			var o = controller.options;

			// warunki początkowe
			if(!o['success']) {
				throw Error("tt-ui-ajax-success musi zostać podane dla ajax-form !");
			}
			
			if(!o['error']) {
				o['error'] = showErrorResponse;
			}
			
			$(iElement).ajaxForm(o);
		}
	};
});
// target (wartość stanowi jquery  id contenera do którego załadować odpowiedź serwera na żądanie)
// jeżeli nic ma się nie zmieniać w widoku to po prostu należy pominąć dyrektywę
ttUI.directive('ttUiAjaxTarget', function() {
	return {
		priority: 500,
		require: '^ttUiAjaxForm',
		compile: function(element, attrs, transclude) {
			return function($scope, iElement, iAttrs, controller) {
				// gdy zdefiniowano id
				if(iAttrs.ttUiAjaxTarget) {
					controller.options['target'] = iAttrs.ttUiAjaxSuccess;
				}
				// generowanie id
				else {
					var id = iElement.attr("id");
					if(id == null) {
						// nadawanie unikalnego
						id = "frm" + new Date().getTime() + '_' + Math.floor((Math.random() * 10000000) + 1);
						iElement.attr("id", id);
					}
					controller.options['target'] = "#" + id;
				}
			};
		}
	};
});
// success callback
ttUI.directive('ttUiAjaxSuccess', function($window) {
	return {
		priority: 500,
		require: '^ttUiAjaxForm',
		compile: function(element, attrs, transclude) {
			return function($scope, iElement, iAttrs, controller) {
				if($scope[iAttrs.ttUiAjaxSuccess]) {
					controller.options['success'] = $scope[iAttrs.ttUiAjaxSuccess];
				}
				else if($window[iAttrs.ttUiAjaxError] != undefined) {
					controller.options['success'] = $window[iAttrs.ttUiAjaxSuccess];
				}
			};
		}
	};
});
//error callback
ttUI.directive('ttUiAjaxError', function($window) {
	return {
		priority: 500,
		require: '^ttUiAjaxForm',
		compile: function(element, attrs, transclude) {
			return function($scope, iElement, iAttrs, controller) {
				if($window[iAttrs.ttUiAjaxError] != undefined) {
					controller.options['error'] = $window[iAttrs.ttUiAjaxError];
				}
			};
		}
	};
});
