!function(root) {
	'use strict';

	var _ = root._;

	var resultOf = function(property) {
		return function(item) {
			return _.result(item, property);
		};
	};

	var filterByResult = function(list, property) {
		return _.filter(list, resultOf(property));
	};

	var sortByResult = function(list, property) {
		return _.sortBy(list, resultOf(property));
	};

	var pluckResult = function(list, property) {
		return _.map(list, resultOf(property));
	};

	var zipmap = function(keys, vals) {
		var map = {};
		for ( var i in keys) {
			map[keys[i]] = vals[i];
		}
		return map;
	};

	var translate = function(chars1, chars2) {
		var map = zipmap(chars1, chars2);
		return function(str) {
			return _.map(str.split(""), function(char) {
				return map[char] || char;
			}).join("");
		};
	};

	var partitionBy = function(list, iterator) {
		var result = [];
		var yprev = undefined;
		for ( var i = 0; i < list.length; i++) {
			var x = list[i];
			var y = iterator(x);
			if (y !== yprev || i == 0) {
				result.push([]);
			}
			_.last(result).push(x);
			yprev = y;
		}
		return result;
	};

	var reduceSum = function(obj) {
		if (!$.isArray(obj) || obj.length == 0) {
			return 0.0;
		}
		return _.reduce(obj, function(sum, n) {
			return sum + n;
		});
	};

	var _tt = {
		resultOf : resultOf,
		filterByResult : filterByResult,
		sortByResult : sortByResult,
		pluckResult : pluckResult,
		zipmap : zipmap,
		translate : translate,
		partitionBy : partitionBy,
		reduceSum : reduceSum
	};

	root._.tt = _tt;
}(this);
