/**
 *  Project: Collapsible Checkbox Tree jQuery Plugin
 *   Version: 1.0.1
 *	Author: Lewis Jenkins
 *	Website: http://www.redcarrot.co.uk/2009/11/11/collapsible-checkbox-tree-jquery-plugin/
 *    
 *    License:
 *        The CheckTree jQuery plugin is currently available for use in all personal or 
 *        commercial projects under both MIT and GPL licenses. This means that you can choose 
 *        the license that best suits your project and use it accordingly.
 * 
 *  W TT wprowadzono drobne zmiany.
 */
(function($) {
 
	$.fn.collapsibleCheckboxTree = function(options) {
		
		var defaults = {
			initialState : 'default' // Options - 'expand' (fully expanded), 'collapse' (fully collapsed) or default
		};
			
		var options = $.extend(defaults, options);
 
		this.each(function() {
			var $root = this;
						   
			// Add button
			$(this).before('<button id="expand">Rozwiń wszystkie</button>&nbsp;&nbsp;<button id="default">Rozwiń zaznaczone</button>&nbsp;&nbsp;<button id="collapse">Zwiń wszystkie</button>');

			// Hide all except top level
			$("ul", $(this)).addClass('hide');

			// Show checked and immediate children of checked
			$("li:has(input:checked) > ul", $(this)).removeClass('hide');
			
			// Add tree links
			$("li", $(this)).prepend('<span>&nbsp;&nbsp;</span>');
			$("li:has(> ul:not(.hide)) > span", $(this)).addClass('expanded');
			$("li:has(> ul.hide) > span", $(this)).addClass('collapsed');
			
			// Checkbox function
			$("input[type='checkbox']", $(this)).click(function() {
				// If checking ...
				if ($(this).is(":checked")) {
					// Show immediate children  of checked
					$("> ul", $(this).parent("li")).removeClass('hide');

					// Update the tree
					$("> span.collapsed", $(this).parent("li")).removeClass("collapsed").addClass("expanded");
					
					// Check children if necessary
					var id = $(this).attr("id");
					//jeżeli id nie jest puste
					if(id.length) {
						id = id.substring(0, 2);
						$(this).parent("li").find("input[type='checkbox'][id^='" + id +"']").attr('checked', true);
					}

					// Show all children of checked
					$("ul", $(this).parent("li")).removeClass('hide');

					// Update the tree
					$("span.collapsed", $(this).parent("li")).removeClass("collapsed").addClass("expanded");
				// If unchecking...
				} else {
					// Uncheck children if necessary
					var id = $(this).attr("id");
					//jeżeli id nie jest puste
					if(id.length) {
						id = id.substring(0, 2);
						$(this).parent("li").find("input[type='checkbox'][id^='" + id +"']").attr('checked', false);
					}
				}
			});

			// Tree function
			$("li:has(> ul) span", $(this)).click(function(){
				// If was previously collapsed...
				if ($(this).is(".collapsed")) {
					// ... then expand
					$("> ul", $(this).parent("li")).removeClass('hide');
					// ... and update the html
					$(this).removeClass("collapsed").addClass("expanded");

				// If was previously expanded...
				} else if ($(this).is(".expanded")) {
					// ... then collapse
					$("> ul", $(this).parent("li")).addClass('hide');
					// and update the html
					$(this).removeClass("expanded").addClass("collapsed");
				}
			});
			
			// Button functions
			
			// Expand all
			$("#expand").click(function () {
				// Show all children			 
				$("ul", $root).removeClass('hide');
				// and update the html
				$("li:has(> ul) > span", $root).removeClass("collapsed").addClass("expanded");
				return false;
			});
			// Collapse all
			$("#collapse").click(function () {
				// Hide all children				   
				$("ul", $root).addClass('hide');
				// and update the html
				$("li:has(> ul) > span", $root).removeClass("expanded").addClass("collapsed");
				return false;
			});
			// Wrap around checked boxes
			$("#default").click(function () {
				// Hide all except top level  
				$("ul", $root).addClass('hide');
				// Show checked and immediate children of checked
				$("li:has(input:checked) > ul", $root).removeClass('hide');
				// and update the html
				$("li:has(> ul:not(.hide)) > span", $root).removeClass('collapsed').addClass('expanded');
				$("li:has(> ul.hide) > span", $root).removeClass('expanded').addClass('collapsed');
				return false;
			});
			
			switch(defaults.initialState) {
				case 'expand':
					$("#expand").trigger('click');
					break;
				case 'collapse':
					$("#collapse").trigger('click');
					break;
				case 'default':
					$("#default").trigger('click');
					break;
			}
		});
		
		return this;
	};
})(jQuery);
