// http://www.swift.com/dsp/resources/documents/IBAN_Registry.pdf

var IBAN_REGISTRY = {
	"AD": 24, "DO": 28, "IS": 26, "NL": 18,
	"AE": 23, "EE": 20, "IT": 27, "NO": 15,
	"AL": 28, "ES": 24, "KW": 30, "PK": 24,
	"AT": 20, "FI": 18, "KZ": 20, "PL": 28,
	"AZ": 28, "FO": 18, "LB": 28, "PS": 29,
	"BA": 20, "FR": 27, "LI": 21, "PT": 25,
	"BE": 16, "GB": 22, "LT": 20, "RO": 24,
	"BG": 22, "GE": 22, "LU": 20, "RS": 22,
	"BH": 22, "GI": 23, "LV": 21, "SA": 24,
	"BR": 29, "GL": 18, "MC": 27, "SE": 24,
	"CH": 21, "GR": 27, "MD": 24, "SI": 19,
	"CR": 21, "GT": 28, "ME": 22, "SK": 24,
	"CY": 28, "HR": 21, "MK": 19, "SM": 27,
	"CZ": 24, "HU": 28, "MR": 27, "TN": 24,
	"DE": 22, "IE": 22, "MT": 31, "TR": 26,
	"DK": 18, "IL": 23, "MU": 30, "VG": 24
};

// http://www.tbg5-finance.org/?ibancheck.shtml

var IBAN_PATTERN = /[A-Z]{2}[0-9]{2}[A-Z0-9]+/;

function parseIntWithBase(b) {
	return function(i) {
		return parseInt(i, b);
	};
}

function isValidIBAN(iban) {
	if ((iban || "").match(IBAN_PATTERN)) {
		var alpha2 = iban.slice(0, 2);
		if (iban.length == IBAN_REGISTRY[alpha2] + 4) {
			var _iban = iban.slice(4) + iban.slice(0, 4);
			var _digits = _.map(_iban, parseIntWithBase(36)).join("");
			var _number = BigInteger.parse(_digits, 10);
			return _number.remainder(97) == 1;
		}
	}
	
	return false;
}

function isValidPLNRB(nrb) {
	console.log(nrb);
	return isValidIBAN("PL" + nrb);
}
